﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Proteus.Domain.Foundation;
using System.Runtime.Serialization;

namespace MyProject.Domain
{
    public class Customer : DomainBase<Customer>
    {
        private IList<Order> _orders;

        public virtual string City { get; set; }

        public virtual string Firstname { get; set; }

        public virtual string Lastname { get; set; }

        public virtual IEnumerable<Order> Orders
        {
            get
            {
                EnsureOrdersInitialized();
                return _orders;
            }
        }

        public virtual string PostalCode { get; set; }

        public virtual string State { get; set; }

        public virtual void RemoveOrder(Order order)
        {
            EnsureOrdersInitialized();

            if (_orders.Contains(order))
                _orders.Remove(order);
        }

        public virtual void AddOrder(Order order)
        {
            EnsureOrdersInitialized();

            if (order.Items.Count() == 0)
                throw new CannotAddOrderException("Cannot add an order without any items.");

            if (_orders.Contains(order))
                throw new OrderAlreadyAddedException(string.Format("Order with Id {0} already added to the customer.", order.Id));

            _orders.Add(order);
        }

        private void EnsureOrdersInitialized()
        {
            if (_orders == null)
                _orders = new List<Order>();
        }

    }
}