using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Proteus.Domain.Foundation;
using System.Runtime.Serialization;

namespace MyProject.Domain
{
    public class Order : DomainBase<Order>
    {
        private IList<OrderItem> _items;

        private decimal _salesTax;

        public virtual decimal Cost
        {
            get
            {
                return CalculateCost();
            }
        }

        public virtual IEnumerable<OrderItem> Items
        {
            get
            {
                EnsureItemsInitialized();
                return _items;
            }
        }

        public virtual decimal SalesTax
        {
            get
            {
                return _salesTax;
            }
        }

        public virtual void AddItem(OrderItem item)
        {
            if (item == null)
                throw new ArgumentNullException("item");

            if (!(item.Quantity > 0))
                throw new CannotAddItemException("Quantity of the Item to be added must be greater than zero.");

            if (!(item.Cost > 0))
                throw new CannotAddItemException("Costs of the Item to be added must be greater than zero.");

            EnsureItemsInitialized();

            if (_items.Contains(item))
            {
                _items.Single(i => i == item).Quantity += item.Quantity;
            }
            else
            {
                _items.Add(item);
            }

        }

        public virtual void ApplySalesTax(decimal taxAmount)
        {
            _salesTax = taxAmount;
        }

        public virtual void RemoveItem(OrderItem item)
        {
            if (item == null)
                throw new ArgumentException("item");

            EnsureItemsInitialized();

            if (_items.Contains(item))
            {
                _items.Single(i => i == item).Quantity -= item.Quantity;
            }

            if (_items.Single(i => i == item).Quantity == 0)
            {
                _items.Remove(item);
            }
        }

        private bool HasItems()
        {
            return _items != null && _items.Count > 0;
        }

        private decimal CalculateCost()
        {
            if (!HasItems())
                return 0m;

            decimal cost = 0m;

            foreach (OrderItem item in _items)
            {
                cost += item.Quantity * item.Cost;
            }

            return cost;
        }

        private void EnsureItemsInitialized()
        {
            if (_items == null)
                _items = new List<OrderItem>();
        }

    }
}
