﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MyProject.Domain;

namespace MyProject.Services
{
    public class OrderShipper : IOrderShippingService
    {
        private IShippingDispatcherService _dispatcher;
        private IWarehouseService _warehouse;
        private IOrderApprovalService _approver;

        /// <summary>
        /// Initializes a new instance of the OrderShipper class.
        /// </summary>
        /// <param name="dispatcher"></param>
        /// <param name="warehouse"></param>
        /// <param name="approver"></param>
        public OrderShipper(IOrderApprovalService approver, IWarehouseService warehouse, IShippingDispatcherService dispatcher)
        {
            _dispatcher = dispatcher;
            _warehouse = warehouse;
            _approver = approver;
        }

        private void DoShippingStuff(Customer customer, Order order)
        {
            var items = _warehouse.GetItemsFor(order);
            _dispatcher.ShipTo(customer, items);

        }

        public void Ship(Customer customer, Order order)
        {
            if (!_approver.IsValidToShip(customer, order))
                throw new CannotShipOrderException(string.Format("Cannot Ship Order with Id: {0} because its not valid.", order.Id));

            DoShippingStuff(customer, order);
        }

    }
}