﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using MyProject.Domain;
using MyProject.Services;
using Rhino.Mocks;

namespace MyProject.PlacingAndShippingOrdersTests
{
    [TestFixture]
    public class When_Calculating_Tax
    {
        [Test]
        public void Can_Lookup_And_Apply_Tax_Rate()
        {
            var item1 = new OrderItem() { Quantity = 1, Cost = 1 };
            var item2 = new OrderItem() { Quantity = 2, Cost = 1.5m };

            var order = new Order();
            order.AddItem(item1);
            order.AddItem(item2);

            var customer = new Customer();
            customer.AddOrder(order);

            var rateLookup = MockRepository.GenerateStub<ITaxRateLookupService>();
            rateLookup.Expect(lu => lu.DetermineTaxRateFor(customer.PostalCode)).Return(0.1d);

            var calculator = new TaxCalculator(rateLookup);

            Assert.That(calculator.Calculate(customer, order), Is.EqualTo(0.4d));
            
        }
    }
}
